""" Extract atomic charges from Bader Charge Analysis output.
Copyright 2019 Simulation Lab
University of Freiburg
Author:
Modified: Lukas Elflein <elfleinl@cs.uni-freiburg.de>
"""

import ase
from parmed import gromacs
import numpy as np
from smamp.insertHbyList import insertHbyList
import warnings

def extract(snapshot_path='snapshot.pdb', top_path='example.top', bader_path='ACF.dat',
	    out_path='bader_charges.csv'):
	""" Execute everything."""
	# Read the atomic structure
	ase_struct = ase.io.read(snapshot_path)

	# Read the toplogy
	with warnings.catch_warnings():  # supress warnings
		warnings.simplefilter("ignore")
		pmd_top = gromacs.GromacsTopologyFile(top_path, parametrize=False)

	implicitHbondingPartners={'CD4':1,'CD3':1,'CA2':2,'CA3':2,'CB2':2,'CB3':2}
	new_ase_struct, new_pmd_top, names, residues = insertHbyList(ase_struct, pmd_top, 
								     implicitHbondingPartners)

	atomic_number = new_ase_struct.get_atomic_numbers()
	bader_charges = np.genfromtxt(bader_path,skip_header=2,skip_footer=4,usecols=4)
	atomic_charges = atomic_number - bader_charges  # this inverts the sign of the bader charges

	results =[]
	for i in range(127):
		results.append((names[i], residues[i], atomic_charges[i],
				atomic_number[i],bader_charges[i]))

	# find the summed charges of carbon atoms and corresponding H atoms
	list_of_C_atoms = ['CD3', 'CD4', 'CA2', 'CA3', 'CB2', 'CB3']
	atom_res_totq = []  # list of [C-atom, residue, total_charge]
	nra = np.array([names,residues,atomic_charges])

	total_charge_results = np.array(results)[:,:4]
	total_charge_results[:,3] = total_charge_results[:,2]

	terminal_names = np.unique(nra[1])
	for term in terminal_names:
		residue = np.where(nra[1] == str(term))
		# print(residue)
		nra_for_one_residue = nra[:,residue[0]]

		# find all carbon atoms and the belonging H atoms
		for c_atom in list_of_C_atoms:
			C_H_positions = np.flatnonzero(np.core.defchararray.find(
							nra_for_one_residue[0], str(c_atom))!=-1)
			C_H_atoms = nra_for_one_residue[:, C_H_positions]
			total_charge = (C_H_atoms[2]).astype(np.float).sum()
			atom_res_totq.append([c_atom, term, total_charge])

			#overwrite the charge of list_of_C_atoms in results with total_charge
			res_position = np.where((total_charge_results[:,0]==c_atom) 
						& (total_charge_results[:,1]==term))
			#print(res_position+(2,))
			total_charge_results[res_position+(3,)] = total_charge

	with open(out_path, 'w') as f:
		f.write("atom,\t residue,\t q"+ '\n')
		for line in total_charge_results:
			f.write(', \t '.join(line[:-1])+'\n')

def main():
	extract()

if __name__ == '__main__':
	main()



""" Change structure with implicit Hydrogen to one with explicitely defined H-atoms.
Copyright 2019 Simulation Lab
University of Freiburg
Author: Johannes Hoermann <johannes.hoermann@imtek.uni-freiburg.de>
Modified: Lukas Elflein <elfleinl@cs.uni-freiburg.de>
"""

import os
import ase.io
import sys
import warnings
import numpy as np
import parmed as pmd

from ase.data import atomic_numbers
from ase.neighborlist import NeighborList
from matscipy.neighbours import neighbour_list
from parmed import gromacs
from smamp.insertHbyList import insertHbyList
from smamp.tools import find
from smamp.tools import read_atom_numbers



def read_input_files():
    """Search for and read input files (with implicit H-atoms)."""
    ase_struct, pmd_top = None, None

    pdb_file = find(path='..', 	folder_keyword='initial_structure', file_keyword='.pdb')[0]
    top_file = find(path='..', 	folder_keyword='initial_structure', file_keyword='.top')[0]

    ase_struct = ase.io.read(pdb_file)
    pmd_struct = pmd.load_file(pdb_file)
    pmd_top = gromacs.GromacsTopologyFile(top_file, parametrize=False)

    # Make sure we actually found everything we need
    if ase_struct is None:
        raise RuntimeError('structure file (.pdb) not found in {}'.format(input_dir))
    if pmd_top is None:
        raise RuntimeError('topology file (.top) not found in {}'.format(input_dir))
    return ase_struct, pmd_struct, pmd_top


def main(implicitHbondingPartners=None):
    """Execute everything."""

    # Read the hydrogen-number table by default
    if implicitHbondingPartners is None:
        implicitHbondingPartners = read_atom_numbers()

    # Read the united-atoms files extracted from the MD-simulation trajectory
    # throws some warnings on angle types, does not matter for bonding info
    with warnings.catch_warnings():
         warnings.simplefilter('ignore')
         ase_struct, pmd_struct, pmd_top = read_input_files()

    pmd_top.strip(':SOL,CL') # strip water and electrolyte from system

    pmd_top.box = pmd_struct.box # Needed because .prmtop contains box info
    pmd_top.positions = pmd_struct.positions

    # Insert the explicit hydrogens
    print('Inserting explicit hydrogens, please wait ...')
    with open('insert_H.log', 'w') as logfile:
       new_ase_struct, new_pmd_top, names, residues = insertHbyList(ase_struct,
                                                                    pmd_top,
                                                                    implicitHbondingPartners,
                                                                    bond_length=1.0, 
                                                                    debug=logfile)

    # Write output
    new_ase_struct.write('ase_pdbH.pdb')
    new_ase_struct.write('ase_pdbH.traj')

    # Write other output
    new_pmd_top.write_pdb('pmd_pdbH.pdb')
    test_pmd = pmd.load_file('pmd_pdbH.pdb')
    # some topology format, un functionality similar to GROMACS' .top, but readable by VMD
    # new_pmd_top.write_psf('pmd_pdbH.psf')
    print('Done.')


if __name__ == '__main__':
    main()



""" Misc tools
Copyright 2019 Simulation Lab
University of Freiburg
Author: Lukas Elflein <elfleinl@cs.uni-freiburg.de>
"""

import os 
import pandas as pd

class cd:
    """Context manager for changing the current working directory"""
    def __init__(self, newPath):
        self.newPath = os.path.expanduser(newPath)

    def __enter__(self):
        self.savedPath = os.getcwd()
        os.chdir(self.newPath)

    def __exit__(self, etype, value, traceback):
        os.chdir(self.savedPath)

def check_existence(path, neccessary_files, verbose=True):
	"""Check if all neccessary files exist in path, print warning for missing files.

	Arguments:
	path: string, the path to search
	neccessary_files: list of strings, check if they exist in path
	verbose: Bool, controls amount of printing

	Returns:
	String warning xor None 
	"""
	with cd(path):
		# Files should be here
		# if verbose:
		#	print([f for s, d, f in os.walk('.')])

		for f in neccessary_files:
			if not (os.path.islink(f) or os.path.isfile(f)):
				warning = 'File not found: {}'.format(f)
				if verbose:
					print(Warning(warning))
				return warning
	return None


def find(path, folder_keyword, file_keyword, nr_occ=1, exclude_kw=['template', 'exclude']):
	"""Search folderstructure for files, return paths containing them.

	Arguments:
	path: string, the path to search
	folder_keyword: strings of the subfolders where the files are supposed to be located
	file_keyword: parts of the filename to be located
	nr_occ: how often the file keyword is supposed to occur in the lowererst folder
	exclude_kw: list of keywords where search should be skipped

	Returns:
	List of strings, a collection of all matching paths

	Example:
	crawl(path='./', folder_keyword='dft', file_keyword='rho', nr_occ=1)
	['./600_ps_snapshot/2_dft_calculations/rho.cube', 
	 './600_ps_snapshot/2_dft_calculations/rho.cube', 
	 ...]
	"""
	paths = []
	# Crawl the directory structure
	for subdir, dirs, files in sorted(os.walk(path)):

		# Exclude template folders from search
		if any([keyword in subdir for keyword in exclude_kw]):
			continue

		# Select the folders with the required keyword in them
		if folder_keyword in subdir:

			if nr_occ is not None:
				# Check if at least one desired file exists
				if sum([file_keyword in f for f in files]) < 1:
					print(os.getcwd())
					raise RuntimeError('No {} file found.'.format(file_keyword))

				# No more than one file must exists for uniqueness
				if sum([file_keyword in f for f in files]) > nr_occ:
					print(os.getcwd())
					err = 'Multiple files containing {} found.'.format(file_keyword)
					raise RuntimeError(err)
						
			# If no error was raised, we can use the file we found:
			for f in files:
				if file_keyword in f:
					paths += [os.path.join(subdir, f)]
	return paths


def read_atom_numbers(path='../fitting_constraint_files/hydrogen_per_atom.csv'):
	"""Determines number of explicit Hydrogen atoms per Carbon from a table.
	Arguments
	path: Path to the table.

	Returns:
	hydrogen_per_atom: dictionary mapping atom names to number of hydrogens to insert.
	"""
	df = pd.read_csv(path)
	df = df.set_index('atom', drop=True)
	# target = {'CD4':1,'CD3':1,'CA2':2,'CA3':2,'CB2':2,'CB3':2}
	hydrogen_per_atom = df.to_dict()[df.columns[0]]

	return hydrogen_per_atom


def read_total_charge(path='../fitting_constraint_files/total_charge.csv'):
	"""Determine total charge of molecule by reading file.
	Arguments
	path: Path to the table.

	Returns:
	charge: a float
	"""
	# The default charge is zero, use this if no file is found
	if not os.path.isfile(path):
		print('WARNING: No charge file found at {}. Using default charge of 0.0'.format(path))
		return 0.0

	with open(path) as charge_file:
		for line in charge_file:
			charge = float(line)
	return charge



## jlh 2018/02/13
# takes dictionary of united atoms and implicit hydrogen number,
# adding a certain number of explicit hydrogens around these.
# Needs and returns corresponding ASE and ParmEd representations of the system
# Ugly and annoying: 
# ASE identifies atoms only by number, no functionality to store atom name and residue 
#  New atoms are added at the end of the whole atom list
# Parmed inserts new atoms not at the end of the list, nut within their respective residues.
# Initially, ordering in ASE and Parmed are equal, but when structure is modified
# we have to track every change in order to be able to map Parmed structure to ase structure

# TODO: think about the case when one has to add more than two H-atoms


import numpy as np
import matscipy as msp
from matscipy.neighbours import neighbour_list
from ase.data import atomic_numbers
import ase.io
from ase.neighborlist import NeighborList
from ase.visualize import view
import parmed as pmd
from parmed import gromacs
import sys

import logging
#gromacs.GROMACS_TOPDIR = "/home/jotelha/gromacs/2016.4/share/gromacs/top"

# returns new_ase_struct and new_mpd_struct with hydrogens added
# additionally, two lists, "names" and "residues" are returned in order to make ASE atoms identifiable
def insertHbyList(ase_struct,pmd_top,implicitHbondingPartners,bond_length=1.0,debug=False):
    # make copies of passed structures as not to alter originals:
    new_pmd_top = pmd_top.copy(pmd.Structure)
    new_ase_struct = ase_struct.copy()

    # names stores the String IDs of all atoms as to facilitate later ASE ID -> Atom name mapping
    names = [ a.name for a in pmd_top.atoms ]
    residues = [ a.residue.name for a in pmd_top.atoms ]

    # make copied atoms accessible by unchangable indices (standard list)
    originalAtoms = [a for a in new_pmd_top.atoms]

    implicitHbondingPartnersIdxHnoTuples = [(a.idx,implicitHbondingPartners[k])
                                            for a in pmd_top.atoms
                                            for k in implicitHbondingPartners.keys()
                                            if a.name == k]
    implicitHbondingPartnersIdxHnoDict = dict(implicitHbondingPartnersIdxHnoTuples)

    # build numbered neighbour list "manually"
    #i: list of atoms e.g. [0,0,0,1,1,2,2,...]
    i = np.array([b.atom1.idx for b in new_pmd_top.bonds])
    #j: list of bonding partners corresponding to i [1,2,3,...] ==> 0 has bondpartners 1,2 and 3; etc.
    j = np.array([b.atom2.idx for b in new_pmd_top.bonds])
    r = new_ase_struct.positions

    for k,Hno in implicitHbondingPartnersIdxHnoDict.items(): # for all atoms to append hydrogen to
        logging.info('Adding {} H-atoms to {} (#{})...'.format(Hno,originalAtoms[k].name,k))
        for h in range(0,Hno):
            r = new_ase_struct.positions
            bondingPartners = j[i==k]
            logging.info('bondingPartners {}'.format(bondingPartners))
            partnerStr=''
            for p in bondingPartners:
                if partnerStr == '':
                    partnerStr = originalAtoms[p].name
                else:
                    partnerStr += ', ' + originalAtoms[p].name
            logging.info('Atom {} already has bonding partners {}'.format(originalAtoms[k].name,partnerStr))
            dr = (r[j[i==k]] - r[k]).mean(axis=0)
            # my understanding: dr is vector
            # from atom k's position towards the geometrical center of mass
            # it forms with its defined neighbours
            # r0 is a vector offset into the opposit direction:
            dr = dr/np.linalg.norm(dr) #normalized vector in direction dr

            #calculate an orthogonal vector 'dr_ortho' on dr
            #and push the H atoms in dr+dr_ortho and dr-dr_ortho
            #if one has to add more than two H atoms introduce dr_ortho_2 = dr x dr_ortho
            dr_ortho = np.cross(dr,np.array([1,0,0]))
            if np.linalg.norm(dr_ortho) < 0.1 :   #if dr and (1,0,0) have almost the same direction
                dr_ortho = np.cross(dr,np.array([0,1,0]))
            # (1-2*h) = {1,-1} for h={0,1}
            h_pos_vec = (dr+(1-2*h)*dr_ortho)/np.linalg.norm(dr+(1-2*h)*dr_ortho)
            r0 = r[k] - bond_length*h_pos_vec

            new_ase_struct += ase.Atom('H', r0) # add atom in ase structure
            n_atoms = len(new_ase_struct)

            #introduce a corrector step for a added atom which is too close to others
            #do as many corrector stepps until all atoms are more than 1\AA appart
            c_step = 0
            while True:
                nl = NeighborList(cutoffs=[.5]*len(new_ase_struct),
                                  skin=0.09, self_interaction=False,
                                  bothways=True)
                nl.update(new_ase_struct)
                indices, offsets = nl.get_neighbors(-1)
                indices = np.delete(indices, np.where(indices==k))
                if len(indices) == 0:
                    break

                elif c_step > 15:
                    logging.info('programm needs more than 15 corrector steps for H atom {} at atom {}'.format(n_atoms, k))
                    sys.exit(15)
                    break

                logging.info('too close atoms {}'.format(indices))
                c_step += 1
                logging.info('correcter step {} for H {} at atom {}'.format(c_step, n_atoms-1, k))
                # if indices not empty -> the atom(-1)=r_H is to close together
                # with atom a_close=indices[0], it is a H-atom belonging to atom 'k'=r_k .
                #correctorstep: corr_step = (r_H-a_close)/|(r_H-a_close)|
                #corrected_pos: corr_pos = ((r_H-r_k) + corr_step)/|((r_H-r_k) + corr_step)|
                #new H position: new_r_H = r_k + corr_pos
                r_H, r_k, a_close = np.take(new_ase_struct.get_positions(),[-1,k,indices[0]], axis=0)
                #print('r_H, r_k, a_close', r_H, r_k, a_close)
                corr_step = (r_H-a_close)/np.linalg.norm((r_H-a_close)) #maybe introduce here a skaling Faktor s=0.3 or somthing like that to make tiny corrections and don't overshoot.
                corr_pos = ((r_H-r_k) + corr_step)/np.linalg.norm((r_H-r_k) + corr_step)
                new_r_H = r_k + bond_length * corr_pos
                #correct the H position to new_r_H in new_ase_struct
                trans = np.zeros([n_atoms,3])
                trans[-1] = new_r_H - r_H
                new_ase_struct.translate(trans)

            #view(new_ase_struct)
            #sys.exit()

            i = np.append(i,k) # manually update numbered neighbour lists
            j = np.append(j,len(new_ase_struct)-1)

            # update pmd topology
            bondingPartner = originalAtoms[k] # here we need the original numbering,
            # as ParmEd alters indices when adding atoms to the structure
            nameH = '{}{}'.format(h+1,bondingPartner.name) # atom needs a unique name
            logging.info('Adding H-atom {} at position [ {}, {}, {} ]'.format(nameH,r0[0], r0[1], r0[2]))
            new_H = pmd.Atom(name=nameH, type='H', atomic_number=1)
            new_H.xx = r0[0] # ParmEd documentation not very helpful, did not find any more compact assignment
            new_H.xy = r0[1]
            new_H.xz = r0[2]
            # do not understand ParmEd that well, apparently we need the Bond object in order to update topology
            new_Bond = pmd.Bond(bondingPartner,new_H)
            new_H.bond_to(bondingPartner) # not sure, whether this is necessary
            new_pmd_top.bonds.append(new_Bond)
            new_pmd_top.add_atom_to_residue(new_H,bondingPartner.residue)
            originalAtoms.append(new_H) # add atom to the bottom of "index-stiff" list
            
            names.append(nameH) # append name of H-atom
            residues.append(bondingPartner.residue.name) # H is in same residue as bonding partner
    return new_ase_struct, new_pmd_top, names, residues



"""SMAMP module."""

# Modules by Johannes
from . import insertHbyList
from . import aa2ua_cube


# Modules by Lukas
from . import tools
from . import convert_UA_to_AA
from . import charges_to_rtp
from . import extract_bader_charges 



#!/usr/bin/env python   
""" Map point charges obtained by GPAW and HORTON on the original GROMACS topology.
Copyright 2019 Simulation Lab
University of Freiburg
Author: Johannes Hoermann <johannes.hoermann@imtek.uni-freiburg.de>
Modified: Lukas Elflein <elfleinl@cs.uni-freiburg.de>
"""

import ast
import h5py
import ase.io
from ase.io.cube import read_cube_data
import parmed as pmd
from parmed import gromacs
from smamp.insertHbyList import insertHbyList

import warnings
import argparse

def main():
    parser = argparse.ArgumentParser(\
        description='Converts an all-atom cube file into united-atom'
            ' representation based on certain replacement rules')
    #parser.add_argument('-c', '--charge',metavar='INTEGER_CHARGE',
    #        type=int,nargs='?', const=1, default=0)
    #parser.add_argument('infile', nargs='?')
    parser.add_argument('infile_pdb', nargs='?', metavar='infile.pdb',
            default='system.pdb',
            help="Original .pdb file, before insertion of implicit hydrogen.")
    parser.add_argument('infile_top', nargs='?', metavar='infile.top',
            default='system.top', help="Original GROMACS .top file")
    parser.add_argument('infile_cube', nargs='?', metavar='infile.cube',
            default='esp.cube',
            help="ESP descrition (or other scalar field) in all-atom cube file.")
    parser.add_argument('outfile_cube', nargs='?', metavar='outfile.cube', 
            default='esp_fitted_system.top', help="Output truncated by atoms only"
            "present in all-atoms description")
    parser.add_argument('-i','--insertion-rules',
            default="{'CD4':1,'CD3':1,'CA2':2,'CA3':2,'CB2':2,'CB3':2}",
            help="A string representation of a python dictionary, describing how "
            "many implicit hydrogens have been inserted at which atom. Example: "
            "{'CD4':1,'CD3':1,'CA2':2,'CA3':2,'CB2':2,'CB3':2}")
    args = parser.parse_args()

    print('Using replacement rules "{}"...'.format(args.insertion_rules))
    #implicitHbondingPartners = ast.literal_eval(args.insertion_rules)
    
    aa2ua_cube(args.infile_pdb, args.infile_top, args.infile_cube, 
               args.outfile_cube,implicitHbondingPartners=implicitHbondingPartners)


def aa2ua_cube(infile_pdb, infile_top, infile_cube,
               outfile_cube, implicitHbondingPartners):

    ase_struct=ase.io.read(infile_pdb)
    pmd_struct = pmd.load_file(infile_pdb)

    with warnings.catch_warnings():
         warnings.simplefilter('ignore')
         # throws some warnings on angle types, does not matter for bonding info
         pmd_top = gromacs.GromacsTopologyFile(infile_top,parametrize=False)

    pmd_top.strip(':SOL,CL') # strip water and electrolyte from system
    pmd_top.box = pmd_struct.box # Needed because .prmtop contains box info
    pmd_top.positions = pmd_struct.positions

    new_ase_struct, new_pmd_struct, names, residues = insertHbyList(
        ase_struct,pmd_top,implicitHbondingPartners,1.0)


    surplus_atoms = len(new_ase_struct) - len(ase_struct)
    # print("{} atoms are going to be truncated from file "
    #       "{} ...".format(surplus_atoms,infile_cube))
    # hdf5 = h5py.File(infile_h5,'r')
    cube_data, cube_atoms = read_cube_data(infile_cube)
    # print("Truncated atoms: {}".format(cube_atoms[len(ase_struct):]))
    ase.io.write(outfile_cube, cube_atoms[0:len(ase_struct)], data=cube_data)
    # ATTENTION: this script just truncates atoms based on total count difference
    # in UA and AA representations

if __name__ == '__main__':
    main()



from setuptools import setup
setup(name='smamp',
      version='1.07',
      description='Self-consistently optimizing smamp charges.',
      packages=['smamp'],
      url='',
      license='MIT',
      author='Lukas Elflein',
      author_email='elfleinl@tf.uni-freiburg.de'
     )



""" Transfer Charges from CSV table to .rft file
Copyright 2019 Simulation Lab
University of Freiburg
Author: Lukas Elflein <elfleinl@cs.uni-freiburg.de>
"""

import pandas as pd
import argparse
import shutil


def import_charges(csv_path):
	"""
	Read and filter the best-fit charges.

	Args:
	csv_path: path to the charge table
	
	Return:
	charges: pandas DataFrame with atom names, residue names, charges
	"""
	charges = pd.read_csv(csv_path)

	# select atom names, the residue and q column
	charges = charges[['atom', 'residue', 'q']]
	print('Charge table sucessfully imported.')

	return charges

def parse_rft(rtp_path, charges=None):
	"""
	Substitute the fitted charges for the original charges in the RTF file.

	Args:
	rtp_path: A string containing the original GROMACS topology file path
	charges: A pandas DataFrame containing the best fit charges

	Returns:
	fitted_rtp_text: The original topoly file, but with updated charges
	"""

	with open(rtp_path, 'r') as rtp_file:
		print('Successfully loaded topolgy file {}'.format(rtp_path))
		rtp_text = rtp_file.readlines()

		# save all possible atom names
		atom_names = charges.atom.unique()
		# and residuum names
		residuum_names = charges.residue.unique()
		
		# We will append all modified lines to sub_text
		sub_text = ''
		
		# Keep track of how often we substituted for debugging
		nr_substitutions = 0

		current_residuum = None		
		for line in rtp_text:
			# Atom names are only unique inside one residuum
			# Thus, specify which res we are currently in
			for residuum in residuum_names:
				if residuum in line:
					current_residuum = residuum
					break
			# Now, we can look up the atom name in the charge table.
			# First, select the lines with exactly one atom name
			for atom_name in atom_names:
				# Select lines with at least one atom name
				if atom_name in line[0:7]:
					second_entry = line[8:18].replace('+', '')
					second_entry = second_entry.replace('-', '').strip()
					# Select lines with no atom name in second column
					if not second_entry in atom_names:
						# Now we can substitute the charge with the fitted one
						line = substitute(line, charges, current_residuum)
						nr_substitutions += 1
						break

			# We substituted the charge, if applicable.
			# Now we can append the line to the text file
			sub_text += line

	# Make sure that all substitutions were executed
	assert len(charges.index) == nr_substitutions
	print('{} of {} charges substituted.'.format(nr_substitutions, len(charges.index)))

	return sub_text

def substitute(line, charges, current_residuum):
	"""
	Look up the charge for an atom, and write into a string.

	Args:
	line: String containing an atom name, residuum name, and original charge.
	charges: DataFrame containing atom name, residuum names, new charges.

	Returns:
	line: the original line, with the charge updated from the `charges` table.
	"""
	current_atom = line[0:7].strip()

	# look up the current atom name in the residuum we are currently in
	mask = (charges.atom == current_atom) & (charges.residue == current_residuum)
	# Exract the charge q
	new_charge = charges[mask].q.values[0]
	charge_string = '{: 1.6f}'.format(new_charge) + '  '
	modified_line = line[:24] + charge_string + line[34:]

	return modified_line

def export_rft(text, out_path):
	"""
	Write the modified text to a file.
	"""
	with open(out_path, 'w') as outfile:
		outfile.write(text)
	print('Modified topoly file written to {}'.format(out_path))

def cmd_parser():
	parser = argparse.ArgumentParser(prog='',
					 description='Transfer charges from a csv file to an .rtp file')
	parser.add_argument('-rtp',
        help='The location of the GROMACS topology file (.rtp)',
	default='./n7nh2.rtp', metavar='./n7nh2.rtp')

	parser.add_argument('-csv', metavar='./charges.csv',
        help='The location of the atomname-charge table, in the .csv format.', 
	default='./average_cost_function_check/fitted_points_charges.csv')
	
	parser.add_argument('-out', metavar='./modified.rtp',
        help='The location where the modified rtp file should be saved.',
	default='modified.rtp')

	args = parser.parse_args()

	return args.rtp, args.csv, args.out
	
	
def main():
	"""
	Run the script.
	"""
	rtp_path, csv_path, out_path = cmd_parser()
	charges = import_charges(csv_path)
	sub_text = parse_rft(rtp_path=rtp_path, charges=charges) 
	export_rft(text=sub_text, out_path=out_path)
	print('Done.')

if __name__ == '__main__':
	main()
